#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
Kysrun application launcher (http://colabti.org/kysrun).
Copyright (C) 2008  Francois Beerten

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

import sys
import os

 # Configuration
dbdir = '~/kysrun/db/'
filetypes = "*.txt *.html *.xls *.ods *.doc *.mp3 *.mpg *.wma *.avi *.wmv"

# Implementation. No need to edit it for configuration.
dbdir = os.path.expanduser(dbdir)

def which(name):
    found = os.popen('which ' + name).readline()
    return found.strip()

for c in ['ksudo', 'gksu', 'gksudo', 'sudo']:
    sudocmd = c
    if which(c):
         break

def run(command):
    print 'Running ' + ' '.join(command)
    os.spawnvp(os.P_NOWAIT, command[0], command)

def savedb(name, db):
    name = dbdir + name
    f = file(name, 'w')
    for i in db:
        f.write(i + '\n')
    f.close()

def loaddb(name):
    db = []
    name = dbdir+name
    if os.access(name, os.R_OK):
         db = [i.strip() for i in file(name).readlines()]
    return db

def findfiles(path, pattern):
    path = os.path.expanduser(path)
    if not os.access(path, os.F_OK):
        return []
    r = os.popen('find -L %s -name "%s"' % (path, pattern)).readlines()
    return [f.strip() for f in r]

class ApplicationProposer:
    def __init__(self):
        self.db = loaddb('apps.dat')
    class ApplicationShortcut:
         def __init__(self, shortcut):
             self.shortcut = shortcut
         def start(self):
             startApplication(self.shortcut)
         def display(self):
             return self.shortcut.split('=')[1]
    def find(self, s):
         return [self.ApplicationShortcut(i) for i in self.db if self.test(i, s)]
    def test(self, shortcut, sub):
         bin, name = shortcut.split('=')
         bin = shortcut.split('=')[0].split('/')[-1].lower()
         sub = sub.lower()
         return sub in name.lower() or sub in bin
    def rebuild(self):
         def isAppSupported(a):
             return not (a.endswith('-kde.desktop') or 'kfmclient' in a or
                        'filebrowser' in a)
         files = findfiles('/usr/share/applications/', '*.desktop')
         files += findfiles('~/.local/share/applications/', '*.desktop')
         self.db = []
         files = filter(isAppSupported, files)
         for f in files:
              d = parseDesktop(f)
              self.db.append(f + '=' + d.get('Name',''))
         konqueror = which('konqueror')
         if konqueror:
             self.db.append('%s=Konqueror File Manager' % konqueror)
         savedb('apps.dat', self.db)

def parseDesktop(desktopfile):
    p = {}
    for line in file(desktopfile):
        if '=' in line:
            k, v = line.split('=', 1)
            p[k] = v.strip()
    return p

def runShortcut(app):
    desktop = parseDesktop(app)
    cmd = desktop['Exec'].split(' ')
    disallowed = ['%f','%F','%u','%U', '%i', '%m', '']
    title = desktop.get('Name', 'Untitled')
    cmd = [c.replace('%c', title) for c in cmd if c not in disallowed]
    if cmd[0] in ['gksu', 'gksudo', 'ksudo', 'sudo']:
        cmd[0] = sudocmd 
    run(cmd)

def startApplication(app):
    if '.desktop' in app:
        app = app.split('.desktop')[0] + '.desktop'
        runShortcut(app)
    else:
        app = app.split('=')[0]
        run(app.split())


class UrlProposer:
    def __init__(self):
        self.db = loaddb('urls.dat')
    class Url:
        def __init__(self, link):
            self.link = link
        def start(self):
            run(['xdg-open', self.link.split(' ')[0]])
        def display(self):
            return self.link.split(' ', 1)[1]
    def find(self, s):
        return [self.Url(i) for i in self.db if self.test(i, s)]
    def test(self, shortcut, sub):
        return sub.lower() in shortcut.split(' ', 1)[1].lower()
    def rebuild(self):
        self.db = []
        files = findfiles('~/.mozilla','bookmarks.html')
        for f in files:
            for s in file(f):
                r = s.split('<A HREF="', 1)
                if len(r) == 2:
                    url, r = r[1].split('"',1)
                    _, name = r.split('>', 1)
                    name = name.split('<',1)[0]
                    self.db.append(url + ' '+ name)
        savedb('urls.dat', self.db)

class FileProposer:
    def __init__(self):
        self.db = loaddb('files.dat')
    class Url:
        def __init__(self, link):
            self.link = link
        def start(self):
            run(['xdg-open', self.link])
        def display(self):
            return self.link.split('/')[-1]
    def find(self, s):
        return [self.Url(i) for i in self.db if self.test(i, s)]
    def test(self, shortcut, sub):
        return sub.lower() in shortcut.split('/')[-1].lower()
    def rebuild(self):
        self.db = []
        for f in filetypes.split(' '):
            self.db += findfiles('~', f)
        savedb('files.dat', self.db)

class WebSearchProposer:
    def __init__(self, key, searchurl):
        self.key = key + ' '
        self.searchurl = searchurl
    class Action:
        def __init__(self, terms, k, u):
            self.terms = terms
            self.searchurl = u
        def start(self):
            terms = self.terms.split(' ',1)[1]
            url = self.searchurl % terms.replace(' ', '+')
            run(['xdg-open', url])
        def display(self):
            return self.terms
    def find(self, s):
        if s.lower().startswith(self.key):
            return [self.Action(s, self.key, self.searchurl)]
        else:
            return []
    def rebuild(self):
        pass

class CalculatorProposer:
    class Action:
        def __init__(self, terms):
            self.terms = terms
        def start(self):
            pass
        def display(self):
            return self.terms
    def find(self, s):
        if s[0].isdigit():
            try:
                r = str(eval(s))
                return [self.Action(r)]
            except:
                pass
        return []
    def rebuild(self):
        pass

class FunctionProposer:
    def __init__(self, function, displayname = None):
        self.function = function
        if displayname:
            self.displayname = displayname
        else:
            self.displayname = function.__name__
    class Action:
        def __init__(self, function, displayname):
            self.function = function
            self.name = displayname
        def start(self):
            self.function()
        def display(self):
            return self.name
    def find(self, s):
        if s.lower() in self.displayname.lower():
            return [self.Action(self.function, self.displayname)]
        return []
    def rebuild(self):
        pass

proposers = [
    ApplicationProposer(), 
    UrlProposer(), 
    FileProposer(), 
    WebSearchProposer('google', 'http://www.google.com/search?q=%s'),
    WebSearchProposer('imdb', 'http://imdb.com/find?s=all&q=%s&x=0&y=0'),
    WebSearchProposer('wikipedia',
       'http://www.wikipedia.org/search-redirect.php?search=%s&language=en'),
    CalculatorProposer(),
    ]

def RefreshKysrunDatabase():
    for p in proposers:
        p.rebuild()
proposers.append(FunctionProposer(RefreshKysrunDatabase, 'Refresh Kysrun Database'))

def setupOnFirstStart():
    if not os.access(dbdir, os.F_OK):
        os.mkdir(dbdir)
        RefreshKysrunDatabase()

